package goods

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"

	"github.com/yl10/kit/guid"
)

// SaveOrders 下单
func (m *GoodsDAO) SaveOrders(order *model.TaGoodsOrders) error {
	if order.UserId == "" {
		return errors.New("没有下单人信息")
	}

	caseID := order.CaseId
	if caseID == "" {
		return errors.New("没有指定案场")
	}

	caseInfo, err := m.getCaseByID(caseID)
	if err != nil {
		return err
	}

	order.OrdersId = guid.NewGUIDString()
	order.OrgId = caseInfo.OrgId
	order.CreateDate = time.Now().Local()
	order.Status = models.STATUS_NORMAL
	order.IsPay = models.BOOL_TRUE // 默认已支付
	order.CouponAmount = "0"

	// 当前订单号的随机方式 = 时间 + 个人ID
	order.OrdersNo = time.Now().Local().Format("20060102150405") + "-" + strings.Join(utils.GUIID2IntString(order.UserId), "")

	if _, err := m.db.Insert(order); err != nil {
		utils.LogError("保存订单主记录失败: " + err.Error())

		return errors.New("保存订单主记录失败")
	}
	return nil
}

// SaveOrdersDetail 保存订单明细
func (m *GoodsDAO) SaveOrdersDetail(list []model.TaGoodsOrdersDetail, ordersID string) error {
	for inx := range list {
		list[inx].OrdersId = ordersID
	}

	if _, err := m.db.Insert(list); err != nil {
		utils.LogError("保存订单明细失败: " + err.Error())

		return errors.New("保存订单明细失败")
	}

	return nil
}

// SaveOrdersCoupon 保存订单优惠券
func (m *GoodsDAO) SaveOrdersCoupon(coupons []model.TaGoodsOrdersCoupon, order *model.TaGoodsOrders) error {
	if order.OrdersId == "" {
		return errors.New("内部错误, 订单事务顺序出错")
	}

	var couponAmount float64 = 0

	// 补充优惠券表字段
	for inx := range coupons {
		coupons[inx].OrdersCouponId = guid.NewGUIDString()
		coupons[inx].OrdersId = order.OrdersId
		coupons[inx].OrdersNo = order.OrdersNo
		coupons[inx].OrgId = order.OrgId
		coupons[inx].Status = models.STATUS_NORMAL
		coupons[inx].CreateDate = time.Now().Local()

		userAmount, _ := strconv.ParseFloat(coupons[inx].UsedAmount, 64)
		couponAmount += userAmount
	}

	// 反向更新 订单主表
	totalAmount, _ := strconv.ParseFloat(order.Amount, 64)

	order.CouponAmount = strconv.FormatFloat(couponAmount, 'f', -1, 32)
	order.ActualAmount = strconv.FormatFloat((totalAmount - couponAmount), 'f', -1, 32)

	if _, err := m.db.Insert(&coupons); err != nil {
		return err
	}

	if _, err := m.db.
		Where("orders_id=?", order.OrdersId).
		Cols([]string{"coupon_amount", "actual_amount"}...).
		Update(&order); err != nil {
		return err
	}

	return nil
}

func (m *GoodsDAO) getCaseByID(caseID string) (*model.SysCase, error) {
	cs := new(model.SysCase)
	if _, err := m.db.Where("case_id=?", caseID).Get(cs); err != nil {
		return nil, err
	}

	return cs, nil
}
